import hashlib
from datetime import datetime
from .base import BaseDatafile


class FileTokens(BaseDatafile):
    def check_token(self, filepath: str, token: str):
        if filepath in self.data:
            return self.data.get(filepath) == token
        return False

    def add_file_token(self, filepath: str) -> str:
        datetime_str = str(datetime.now())
        token_prepare = f"{filepath}||{datetime_str}"
        hash_object = hashlib.sha1(bytes(token_prepare, "utf-8"))
        hash_str = hash_object.hexdigest()
        self.data[filepath] = hash_str
        return hash_str

    def remove_file_token(self, filepath: str) -> bool:
        if filepath in self.data:
            del self.data[filepath]
            return True
        return False

    def remove_file_tokes_beginig(self, filepath_beging: str) -> bool:
        items_to_delete = []
        for path, token in self.data.items():
            if path.startswith(filepath_beging):
                items_to_delete.append(path)
        for path in items_to_delete:
            del(self.data[path])
        return len(items_to_delete) > 0



import os
import sys
from argparse import ArgumentParser

from .core.setuptools import get_file_content
from .core.config import Config
from .core.resources.web import run_web


def _handle_args():
    version = get_file_content(os.path.join(os.path.dirname(__file__), "VERSION"))
    parser = ArgumentParser(description="Share Files WEB UI v{version}".format(version=version))
    parser.add_argument("share_directory", type=str, help="Directory where shares are stored.", nargs="?")
    parser.add_argument("-o", "--host", dest="host", default="0.0.0.0", type=str, help="APP server host")
    parser.add_argument("-p", "--port", dest="port", default=5000, type=int, help="APP server port")
    parser.add_argument("-u", "--add-user", dest="add_user", type=str, help="Add admin user in user@password format.")
    parser.add_argument("-r", "--remove-user", dest="remove_user", type=str, help="Remove admin user from users list.")
    parser.add_argument("-l", "--list-users", dest="list_users", default=False, action="store_true", help="List existing admin users")
    parser.add_argument("-t", "--list-tokens", dest="list_tokens", default=False, action="store_true", help="List file tokens into stdout")
    parser.add_argument("-c", "--clear-tokens", dest="clear_tokens", default=False, action="store_true", help="Clear file tokens database")

    args = parser.parse_args()
    args.__setattr__("version", version)

    return args


def main():
    app_args = _handle_args()

    # config directory handling
    home = os.path.expanduser("~")
    app_config_dir = os.path.join(home, ".sharefile-webui")
    if not os.path.exists(app_config_dir):
        os.makedirs(app_config_dir, exist_ok=True)

    # config
    Config.init(app_args, app_config_dir)

    # handle args
    if app_args.add_user:
        try:
            user: str = app_args.add_user.split("@")[0].strip()
            password: str = app_args.add_user.split("@")[1].strip()
            Config.USERS.add_user(user, password)
            Config.USERS.save_json()
            print(f"Admin user '{user}' added sucessfuly")
            return
        except Exception as ex:
            print(f"ERROR: Unable to parse user@password format: {ex}")
            sys.exit(1)
    elif app_args.remove_user:
        if Config.USERS.remove_user(app_args.remove_user):
            Config.USERS.save_json()
            print(f"User '{app_args.remove_user}' has been removed")
        else:
            print(f"User '{app_args.remove_user}' is not exist")
        return 
    elif app_args.list_users:
        for user in Config.USERS.data.keys():
            print(user)
        return
    elif app_args.list_tokens:
        token_path_list: list = list(Config.FILE_TOKENS.data.keys())
        token_path_list.sort()
        for token_path in token_path_list:
            print(f"{Config.FILE_TOKENS.data[token_path]}\t{token_path}")
        return
    elif app_args.clear_tokens:
        Config.FILE_TOKENS.data = {}
        Config.FILE_TOKENS.save_json()
        print("All tokens has been removed")
        return

    if not app_args.share_directory:
        print("ERROR: Share directory must be specified")
        sys.exit(1)

    run_web(app_args.host, app_args.port)


if __name__ == '__main__':
    main()



import os
from datetime import datetime
from requests.utils import quote
from .base import BaseResource
from ..web import app_auth
from .file_content import FileContent


class Dir(BaseResource):
    RESOURCE_URL = "/api/dir/<path:path>"

    def __init__(self):
        super().__init__()
        self._add_argument("rename", str, "New filename for directory rename")

    @app_auth.login_required
    def get(self, path: str) -> dict:
        """
        List of directory files and subdirectories
        :param path: Direcotory to be listed
        """
        path = self._unquote_path(path)
        result_list_files = []
        result_list_dirs = []
        dir_path = os.path.join(self.root_path, path)
        if not os.path.exists(dir_path):
            self._abort_error(f"Directory '{dir_path}' does not exists.")

        try:
            dir_list: list = os.listdir(dir_path)
        except Exception as ex:
            self._abort_error(str(ex))

        dir_list.sort()

        if path:
            result_list_dirs.append({
                "type": "dir",
                "name": "/",
                "path": "",
            })
            result_list_dirs.append({
                "type": "dir",
                "name": "..",
                "path": os.path.sep.join(path.split(os.path.sep)[:-1]),
            })
        for dir_item in dir_list:
            full_path = os.path.join(dir_path, dir_item)
            context_item_path = os.path.join(path, dir_item)
            mdatetime = os.path.getmtime(full_path)
            if os.path.isfile(full_path):
                result_list_files.append({
                    "type": "file",
                    "name": dir_item,
                    "path": quote(context_item_path),
                    "size": os.path.getsize(full_path),
                    "mdatetime": mdatetime,
                    "mdatetimeISO": datetime.fromtimestamp(mdatetime).strftime("%Y-%m-%d %H:%M"),
                    "token": self.config.FILE_TOKENS.data.get(context_item_path, None),
                    "isEditable": FileContent.check_file_ext(full_path),
                })
            elif os.path.isdir(full_path):
                result_list_dirs.append({
                    "type": "dir",
                    "name": dir_item,
                    "path": quote(context_item_path),
                    "pathLabel": context_item_path,
                    "mdatetime": mdatetime,
                    "mdatetime-iso": datetime.fromtimestamp(mdatetime).strftime("%Y-%m-%d %H:%M"),
                })
        return {
            "status": True,
            "context": quote(path),
            "contextLabel": path,
            "result": result_list_dirs + result_list_files
        }

    @app_auth.login_required
    def delete(self, path: str) -> dict:
        path = self._unquote_path(path)
        full_path = os.path.join(self.root_path, path)
        try:
            os.rmdir(full_path)
        except Exception as ex:
            self._abort_error(f"Dir.delete: {ex}")
        return {
            "status": True
        }

    @app_auth.login_required
    def post(self, path: str) -> dict:
        """
        Create new directory. If exists it will add number suffix
        :param path: New directory path
        """
        path = self._unquote_path(path)
        full_path: str = os.path.join(self.root_path, path)
        dir_tail: str = ""
        tail_counter: int = 1
        while os.path.exists(full_path_tail := f"{full_path}{dir_tail}"):
            dir_tail = f" {tail_counter}"
            tail_counter += 1
        os.mkdir(full_path_tail)
        return {
            "status": True,
            "dir": full_path_tail
        }

    @app_auth.login_required
    def put(self, path: str) -> dict:
        """
        Rename existing directory. New name comes in `rename` GET param
        :param path: Path to existing directory
        """
        path = self._unquote_path(path)
        rename = self.args.get("rename")
        if rename:
            path__1 = os.path.sep.join(path.split(os.path.sep)[:-1])
            full_path = os.path.join(self.root_path, path)
            full_path_rename = os.path.join(self.root_path, path__1, rename)
            if not self._check_path_perms(full_path_rename):
                self._abort_error(f"Permission denied to rename file '{full_path}' to '{full_path_rename}'")
            try:
                os.rename(full_path, full_path_rename)
                # delete all tokens under prevoius path name
                if self.config.FILE_TOKENS.remove_file_tokes_beginig(path):
                    self.config.FILE_TOKENS.save_json()
            except Exception as ex:
                self._abort_error(f"Dir.update: {ex}")
        return {
            "status": True,
            "dir": full_path_rename
        }


class DirRoot(Dir):
    RESOURCE_URL = "/api/dir/"

    @app_auth.login_required
    def get(self) -> dict:
        return super().get("")



from werkzeug.security import generate_password_hash, check_password_hash
from .base import BaseDatafile


class Users(BaseDatafile):
    def add_user(self, user: str, password: str):
        self.data[user] = generate_password_hash(password)

    def remove_user(self, user: str) -> bool:
        if user in self.data:
            del self.data[user]
            return True
        return False

    def check_user(self, user: str, password: str):
        password_hash = self.data.get(user)
        if password_hash:
            check = check_password_hash(password_hash, password)
            return user if check else None



import os
from flask_restful import Resource, reqparse, abort
from requests.utils import unquote
from ...config import Config


class BaseResource(Resource):
    class ArgumentLocation:
        ARGS = "args"
        FORM = "form"

    RESOURCE_URL = None

    def __init__(self):
        self.config: Config = Config
        self.root_path: str = self.config.SHARE_DIRECTORY
        self.parser: reqparse.RequestParser = reqparse.RequestParser()
        self.args: dict = None

    def _add_argument(self, name: str, type_: type = None, help_: str = None, location_: str = ArgumentLocation.ARGS):
        self.parser.add_argument(name, type=type_, help=help_, location=location_)
        self.args = self.parser.parse_args()

    def _check_path_perms(self, path: str) -> bool:
        rpath = os.path.realpath(path)
        return rpath.startswith(self.root_path)

    @staticmethod
    def _unquote_path(path):
        return unquote(path)

    @staticmethod
    def _get_file_ext(path):
        return os.path.splitext(path)[-1].lower().replace(".", "")

    @staticmethod
    def _abort_error(message: str):
        abort(400, message=message, status=False)



import os
from .base import BaseResource
from ..web import app_auth


class File(BaseResource):
    RESOURCE_URL = "/api/file/<path:path>"

    def __init__(self):
        super().__init__()
        self._add_argument("rename", str, "New filename for directory rename")

    @app_auth.login_required
    def delete(self, path: str):
        path = self._unquote_path(path)
        if self.config.FILE_TOKENS.remove_file_token(path):
            self.config.FILE_TOKENS.save_json()
        full_path = os.path.join(self.root_path, path)
        os.remove(full_path)
        return {
            "status": True
        }

    @app_auth.login_required
    def put(self, path: str) -> dict:
        """
        Rename existing file. New name comes in `rename` GET param
        :param path: Path to existing file
        """
        path = self._unquote_path(path)
        rename = self.args.get("rename")
        if rename:
            path__1 = os.path.sep.join(path.split(os.path.sep)[:-1])
            full_path = os.path.join(self.root_path, path)
            full_path_rename = os.path.join(self.root_path, path__1, rename)
            if not self._check_path_perms(full_path_rename):
                self._abort_error(f"Permission denied to rename file '{full_path}' to '{full_path_rename}'")
            try:
                os.rename(full_path, full_path_rename)
                # delete all tokens under prevoius path name
                if self.config.FILE_TOKENS.remove_file_token(path):
                    self.config.FILE_TOKENS.save_json()
            except Exception as ex:
                self._abort_error(f"File.update: {ex}")
        return {
            "status": True,
            "file": full_path_rename
        }



from .base import BaseResource
from ..web import app_auth


class FileToken(BaseResource):
    RESOURCE_URL = "/api/token/<path:path>"

    @app_auth.login_required
    def post(self, path: str):
        path = self._unquote_path(path)
        token = self.config.FILE_TOKENS.add_file_token(path)
        self.config.FILE_TOKENS.save_json()
        return {
            "status": True,
            "token": token
        }

    @app_auth.login_required
    def delete(self, path: str):
        path = self._unquote_path(path)
        self.config.FILE_TOKENS.remove_file_token(path)
        self.config.FILE_TOKENS.save_json()
        return {
            "status": True
        }



def get_file_content_as_list(file_path: str) -> list:
    with open(file_path, "r", encoding="utf-8") as file:
        content = [line.strip() for line in file]
    return content


def get_file_content(file_path: str) -> str:
    with open(file_path, "r", encoding="utf-8") as file:
        content = file.read()
    return content


import markdown
import os
import re
from .base import BaseResource
from ..web import app_auth


class FileContent(BaseResource):
    class AllowedExtension:
        TXT = "txt"
        MD = "md"
        LOG = "log"

    RESOURCE_URL = "/api/filecontent/<path:path>"
    ALLOWED_EXTENSIONS = [AllowedExtension.TXT, AllowedExtension.MD, AllowedExtension.LOG]
    MAX_FILE_SIZE = 64 * 1024

    def __init__(self):
        super().__init__()
        self._add_argument("content", str, "File content to store", location_=self.ArgumentLocation.FORM)
        self._add_argument("decode-content", bool, "Decode content into HTML", location_=self.ArgumentLocation.ARGS)

    @app_auth.login_required
    def get(self, path):
        decode_content = self.args.get("decode-content")
        path = self._unquote_path(path)
        full_path = os.path.join(self.root_path, path)
        if not self.check_file_ext(full_path):
            self._abort_error("Unsupported file extension to get file content")
        if (filesize := os.path.getsize(full_path)) > self.MAX_FILE_SIZE:
            self._abort_error(f"File size reached max size {filesize} > {self.MAX_FILE_SIZE} Bytes")
        with open(full_path, "r") as f:
            content = f.read()
        return {
            "status": True,
            "content": self._decode_content(self._get_file_ext(path), content) if decode_content else content,
            "name": os.path.basename(path),
            "path": path,
            "fullpath": full_path
        }

    @app_auth.login_required
    def post(self, path):
        path = self._unquote_path(path)
        full_path = os.path.join(self.root_path, path)
        file_tail: str = ""
        tail_counter: int = 1
        while os.path.exists(full_path_tail := f"{full_path}{file_tail}.txt"):
            file_tail = f" {tail_counter}"
            tail_counter += 1
        os.mknod(full_path_tail)
        return {
            "status": True,
            "fullpath": full_path_tail
        }

    @app_auth.login_required
    def put(self, path):
        path = self._unquote_path(path)
        full_path = os.path.join(self.root_path, path)
        if not self.check_file_ext(full_path):
            self._abort_error("Unsupported file extension to get file content")

        content = self.args.get("content")
        if content is not None:
            with open(full_path, "w") as f:
                f.write(content)
            return {
                "status": True,
                "file": full_path,
            }
        self._abort_error(f"Content of request for file '{path}' is empty.")

    @classmethod
    def check_file_ext(cls, path: str) -> bool:
        return cls._get_file_ext(path) in cls.ALLOWED_EXTENSIONS

    @classmethod
    def _decode_content(cls, file_extension: str, content: str) -> str:
        if file_extension in (cls.AllowedExtension.TXT, cls.AllowedExtension.LOG):
            # replace URLs in text by links
            url_list = re.findall(r'(https?://[^\s]+)', content)
            for url in url_list:
                content = content.replace(url, f"<a href=\"{url}\">{url}</a>")
            # replace new lines as BR tags
            content = content.replace("\n", "<br/>")
            return content
        elif file_extension == cls.AllowedExtension.MD:
            return markdown.markdown(content, extensions=['fenced_code', 'codehilite'])
        return content



import json
import os


class BaseDatafile:
    def __init__(self, datafile_path: str):
        self._datafile_path: str = datafile_path
        self.data: dict = {}
        self.load_json()

    def load_json(self) -> bool:
        if os.path.exists(self._datafile_path):
            with open(self._datafile_path, "r") as f:
                self.data = json.load(f)
            return True
        return False

    def save_json(self):
        with open(self._datafile_path, "w") as f:
            json.dump(self.data, f)



import os
from argparse import Namespace
from .datafiles.users import Users
from .datafiles.file_tokens import FileTokens


class Config:
    VERSION = ""
    SHARE_DIRECTORY = ""
    USERS_DATAFILE_PATH = ""
    FILETOKENS_DATAFILE_PATH = ""
    USERS: Users = None
    FILE_TOKENS: FileTokens = None

    @classmethod
    def init(cls, app_args: Namespace, app_config_dir: str):
        cls.VERSION = app_args.version
        cls.SHARE_DIRECTORY = app_args.share_directory
        cls.USERS_DATAFILE_PATH = os.path.join(app_config_dir, "users.json")
        cls.FILETOKENS_DATAFILE_PATH = os.path.join(app_config_dir, "files.json")

        cls.USERS = Users(cls.USERS_DATAFILE_PATH)
        cls.FILE_TOKENS = FileTokens(cls.FILETOKENS_DATAFILE_PATH)



import os
from flask import Flask, Response, request, render_template, send_from_directory
from flask_httpauth import HTTPBasicAuth
from flask_restful import Api
from ..datafiles.file_tokens import FileTokens
from ..config import Config


templates_path = os.path.join(os.path.dirname(__file__), "../../templates/")
static_path = os.path.join(os.path.dirname(__file__), "../../static/")

app = Flask(__name__, template_folder=templates_path, static_folder=static_path)
app_auth = HTTPBasicAuth()


def run_web(host: str, port: int):
    from .api.dir import Dir, DirRoot
    from .api.disk import Disk
    from .api.file import File
    from .api.file_content import FileContent
    from .api.file_token import FileToken
    from .api.file_pload import FileUpload, FileUploadRoot

    with app.app_context():
        api = Api(app)
        api.add_resource(Dir, Dir.RESOURCE_URL)
        api.add_resource(DirRoot, DirRoot.RESOURCE_URL)
        api.add_resource(Disk, Disk.RESOURCE_URL)
        api.add_resource(File, File.RESOURCE_URL)
        api.add_resource(FileContent, FileContent.RESOURCE_URL)
        api.add_resource(FileToken, FileToken.RESOURCE_URL)
        api.add_resource(FileUpload, FileUpload.RESOURCE_URL)
        api.add_resource(FileUploadRoot, FileUploadRoot.RESOURCE_URL)
        app.run(host=host, port=port)


@app_auth.verify_password
def verify_password(username, password):
    users = Config.USERS
    return users.check_user(username, password)


@app.route("/", methods=["GET"])
@app_auth.login_required
def index() -> Flask.response_class:
    data = {
        "version": Config.VERSION,
        "user": app_auth.current_user()
    }
    return render_template("index.html", data=data)


@app.route("/share/<path:path>", methods=["GET"])
def share(path) -> Flask.response_class:
    args = request.args
    token = args.get("token", None)
    file_tokens: FileTokens = Config.FILE_TOKENS
    if path not in file_tokens.data:
        return f"File '{path}' not found", 404
    if file_tokens.check_token(path, token):
        full_path = os.path.join(Config.SHARE_DIRECTORY, path)
        if os.path.exists(full_path):
            return send_from_directory(os.path.dirname(full_path), os.path.basename(full_path))
        else:
            return f"404 File '{path}' not found.", 404
    return f"401 Unauthorized access to file '{path}'", 401


@app.route("/sharedir/<path:path>", methods=["GET"])
def sharedir(path) -> Flask.response_class:
    data = {}
    return render_template("index.html", data=data)


@app.route("/svgfilter/<path:path>", methods=["GET"])
def svg_filter(path) -> Flask.response_class:
    args = request.args
    find: str = args.get("find", None)
    replace: str = args.get("replace", None)
    full_path: str = os.path.join(static_path, path)
    content: str = None
    if os.path.splitext(path)[-1].lower() != ".svg":
        return f"406 Not Acceptable file extension in path '{path}'", 406
    if os.path.exists(full_path):
        with open(full_path, "r") as f:
            content = f.read()
        if find and replace:
            content = content.replace(find, replace)
    else:
        return f"404 File '{path}' not found.", 404

    return Response(content, mimetype="image/svg+xml")



import os
import subprocess
from setuptools import setup, find_packages
from sharefile_webui.core.setuptools import get_file_content, get_file_content_as_list

packages = find_packages()
print("Found packages: {packages}".format(packages=packages))
VERSION = get_file_content("sharefile_webui/VERSION")
INSTALL_REQUIRES = get_file_content_as_list("requirements.txt")
DOCUMENTATION_MD = get_file_content("README.md")


def _post_install():
    # rename old name of app settings directory
    home = os.path.expanduser("~")
    old_app_config_dir = os.path.join(home, ".fileshare")
    new_app_config_dir = os.path.join(home, ".sharefile-webui")
    print(f"**** Migration to >1.5.11 - rename old name '{old_app_config_dir}' of app settings directory to '{new_app_config_dir}'.")
    subprocess.run(f"""if [ -d  '{old_app_config_dir}' ]; then mv {old_app_config_dir} {new_app_config_dir}; echo 'RENAMED'; else echo 'SKIPPED'; fi""", shell=True)


def _post_install_wrapper(_setup):
    def _post_actions():
        _post_install()
    _post_actions()
    return _setup


setup = _post_install_wrapper(setup(
        name="sharefile-webui",
        version=VERSION,
        license="MIT",
        author="Ales Adamek",
        author_email="alda78@seznam.cz",
        description="WEB APP for sharing files via URL links.",
        long_description=DOCUMENTATION_MD,
        long_description_content_type="text/markdown",
        url="https://gitlab.com/alda78/sharefile-webui",
        packages=packages,
        classifiers=[
            "Programming Language :: Python :: 3",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
        ],
        python_requires=">=3.8",
        install_requires=INSTALL_REQUIRES,
        include_package_data=True,  # MANIFEST.in
        zip_safe=False,  # aby se spravne vycitala statika pridana pomoci MANIFEST.in
        entry_points={
            "console_scripts": [
                "sharefile-webui=sharefile_webui.main:main",
            ],
        },
    )
)



import os
from flask_restful import request
from .base import BaseResource
from ..web import app_auth


class FileUpload(BaseResource):
    RESOURCE_URL = "/api/upload/<path:path>"

    @app_auth.login_required
    def post(self, path: str):
        path = self._unquote_path(path)
        return self.post_chunk_upload(path)

    def post_chunk_upload(self, path: str):
        chunk_offset: int = int(request.form.get("dzchunkbyteoffset"))
        file: dict = request.files.get("file", {})
        file_path: str = os.path.join(self.root_path, path, file.filename)
        with open(file_path, 'ab') as f:
            f.seek(chunk_offset)
            chunk = file.stream.read()
            f.write(chunk)
        return {
            "status": True,
            "chunkSize": len(chunk),
            "chunkNumber": int(request.form.get("dzchunkindex")),
            "chunkTotalNumber": int(request.form.get("dztotalchunkcount")),
            "currentRemoteFileSize": os.path.getsize(file_path),
            "totalFileSize": int(request.form.get("dztotalfilesize")),
            "filename": file.filename,
            "remotePath": file_path
        }

    def post_multiupload(self, path: str):
        uploaded = []
        files = request.files
        for file_item in files.items():
            _, file_storage = file_item
            filename = file_storage.filename
            file_path = os.path.join(self.config.SHARE_DIRECTORY, path, filename)
            file_storage.save(file_path)
            uploaded.append(filename)
        return {
            "status": True,
            "filename": uploaded,
        }


class FileUploadRoot(FileUpload):
    RESOURCE_URL = "/api/upload/"

    @app_auth.login_required
    def post(self):
        return super().post("")



import shutil
from .base import BaseResource
from ..web import app_auth


class Disk(BaseResource):
    RESOURCE_URL = "/api/disk/"

    @app_auth.login_required
    def get(self):
        usage = shutil.disk_usage(self.root_path)
        return {
            "status": True,
            "diskUsage": {
                "total": usage.total,
                "free": usage.free,
                "used": usage.used,
            }
        }
